from display.color import *

from feature.drawablefeature import *
from feature.bindnfetcher import *
from feature.cddfetcher import *

from util.preferences import *

import wx
import math
import os

class ApplicationHelper:
    def round(self, x):
        return int(float(x) + 0.5)
    
    def floor(self, x):
        return int(x)
    
    def ceil(self, x):
        return int(math.ceil(x))
    
    def getDefaultAlignmentDirectory(self):
        return os.getenv("DATAPATH", os.getcwd())
    
    def determineFeaturesAtLocation(self, features, location, type = None):
        for feat in features:
            if type is not None and feat.type != type:
                continue
            
            if feat.determineAtLocation(location) & DrawableFeature.LOCATION_AT:
                return True
            
        return False
    
    def getAppropriateTextColorForBackground(self, color):
        color = color.toHSV()
        
        if color.getV() >= 0.5:
            return Color(Color.RGB, 0, 0, 0)
        else:
            return Color(Color.RGB, 1, 1, 1)
        
    def getFileExtension(self, filename):
        parts = filename.rsplit('.', 2)
        return parts[1]
    
    # Interpolate colors from a small array to a bigger array
    # Basically a reverse mipmap
    # We need this because OpenGL's GL_LINEAR interpolate has unintended
    # side effects that I don't want.  As of now, I can't find a different way
    # around this
    def interpolateColorsToResolution(self, colorArray, newResolution):
        oldResolution = len(colorArray)
        ratio = float(oldResolution - 1) / (newResolution - 1)
        
        newArray = []
        for i in range(newResolution):
            oldCoord = i * ratio
            
            leftCoord = int(max(0, math.floor(oldCoord)))
            rightCoord = int(min(oldResolution - 1, math.ceil(oldCoord)))
            
            leftColor = colorArray[leftCoord].getRGB()
            rightColor = colorArray[rightCoord].getRGB()
            
            newColor = None
            # Avoid divide by zero, though I don't think this should happen
            # no matter what
            if leftCoord == rightCoord:
                newColor = leftColor
            else:
                newColor = []
                for j in range(3):
                    # Linear interpolation
                    newColor.append(leftColor[j] + (((oldCoord - leftCoord) * (rightColor[j] - leftColor[j])) / (rightCoord - leftCoord)))
                    
            newArray.append(
                            Color(
                                  Color.RGB,
                                  newColor[0],
                                  newColor[1],
                                  newColor[2]
                              )
                        )
            
        return newArray    

appHelper = None
def getApplicationHelper():
    global appHelper
    if appHelper is None:
        appHelper = ApplicationHelper()
        
    return appHelper

app = None
def setApplication(application):
    global app
    app = application
    
def getApplication():
    global app
    return app

def getApplicationFrame():
    return getApplication().getMainFrame()

# Ref: http://yergler.net/projects/stext/stext_py.txt
# StaticWrapText is under the GPL2
class StaticWrapText(wx.StaticText):
    """A StaticText-like widget which implements word wrapping."""
    
    def __init__(self, *args, **kwargs):
        wx.StaticText.__init__(self, *args, **kwargs)

        # store the initial label
        self.__label = super(StaticWrapText, self).GetLabel()

        # listen for sizing events
        self.Bind(wx.EVT_SIZE, self.OnSize)

    def SetLabel(self, newLabel):
        """Store the new label and recalculate the wrapped version."""
        self.__label = newLabel
        self.__wrap()

    def GetLabel(self):
        """Returns the label (unwrapped)."""
        return self.__label
    
    def __wrap(self):
        """Wraps the words in label."""
        words = self.__label.split()
        lines = []

        # get the maximum width (that of our parent)
        max_width = self.GetParent().GetVirtualSizeTuple()[0]
        
        index = 0
        current = []

        for word in words:
            current.append(word)

            if self.GetTextExtent(" ".join(current))[0] > max_width:
                del current[-1]
                lines.append(" ".join(current))

                current = [word]

        # pick up the last line of text
        lines.append(" ".join(current))

        # set the actual label property to the wrapped version
        super(StaticWrapText, self).SetLabel("\n".join(lines))

        # refresh the widget
        self.Refresh()
        
    def OnSize(self, event):
        # dispatch to the wrap method which will 
        # determine if any changes are needed
        self.__wrap()
        
        event.Skip()      
        
class Zoomable():
    def __init__(self, zoom = None):
        if zoom is not None:
            self.setZoom(zoom[0], zoom[1], True)
        else:
            self.setZoom(None, None, True)
            
    def setZoom(self, start, end, fromApplication = False):
        """ Sets the zoom from start to end """
        self.start = start
        self.end = end
        
    def zoomChanged(self, start, end):
        return self.start != start or self.end != end

    def getZoomLength(self):
        return self.end - self.start
    
class SequenceSwapable():
    def __init__(self, seqOrder = None):
        self.seqOrder = seqOrder
        
    def swapSequenceOrder(self, one, two, fromApplication = False):
        valueOne = self.seqOrder[one]
        valueTwo = self.seqOrder[two]
        
        self.seqOrder[one] = valueTwo
        self.seqOrder[two] = valueOne
        
    def orderChanged(self, one, two):
        return (one != two)

class SequenceNotifiable():
    def __init__(self):
        pass
    
    def setNotifySequence(self, index, fromApplication = False):
        pass
    
class RangeSlider(wx.Panel):
    def __init__(self, parent, id, value = 0.000, minValue = 0.000, maxValue = 1.000, pos = wx.DefaultPosition, size = wx.DefaultSize, style = wx.SL_HORIZONTAL | wx.SL_SELRANGE | wx.SL_AUTOTICKS | wx.SL_LABELS, validator = wx.DefaultValidator, name = wx.SliderNameStr, selectInit = True, useLogs = False, labelFormat = "%.3e", segments = None):
        wx.Panel.__init__(self, parent, id, pos, size)
        
        preferences = getPreferences()
        
        self.__useLogs = useLogs
        self.__labelFormat = labelFormat
        
        self.__segments = segments
        if self.__segments is None:
            self.__segments = preferences.getFloatRangeSizerMaxValue() 
        
        # We're going to run our own labels, so note if we want them, but don't pass them on to wx.Slider
        self.__labels = False
        if style & wx.SL_LABELS:
            self.__labels = True
            style = style & ~wx.SL_LABELS

        self.floatMin = minValue
        self.logMin = math.log(self.floatMin)
        self.floatMax = maxValue
        self.logMax = math.log(self.floatMax)

        self.sizer = wx.BoxSizer(wx.VERTICAL)
        
        self.slider = wx.Slider(self, -1, 0, 0, self.__segments, pos, size, style, validator, name)
        
        self.selecting = False
        self.previousStart = self.slider.GetValue()
        if selectInit:
            self.slider.SetSelection(self.slider.GetMin(), self.slider.GetMax())

        self.slider.Bind(wx.EVT_LEFT_DOWN, self.OnLeftDown)
        self.Bind(wx.EVT_LEFT_DOWN, self.OnLeftDown)
        
        self.slider.Bind(wx.EVT_LEFT_UP, self.OnLeftUp)
        self.Bind(wx.EVT_LEFT_UP, self.OnLeftUp)
            
        self.slider.Bind(wx.EVT_MOTION, self.OnMotion)
        self.Bind(wx.EVT_MOTION, self.OnMotion)
        
        self.slider.Bind(wx.EVT_SCROLL_CHANGED, self.OnSliderChange)
        
        self.labelSizer = wx.GridSizer(cols = 3)
        
        self.minLabel = wx.StaticText(self, -1, label = self.formatLabel(self.floatMin), style = wx.ALIGN_LEFT)
        self.maxLabel = wx.StaticText(self, -1, label = self.formatLabel(self.floatMax), style = wx.ALIGN_RIGHT)
        self.valueLabel = wx.StaticText(self, -1, label = self.formatLabel(self.getFloatValue()), style = wx.ALIGN_CENTER)
        
        self.labelSizer.Add(self.minLabel, 1, wx.EXPAND | wx.ALIGN_LEFT)
        self.labelSizer.Add(self.valueLabel, 1, wx.EXPAND | wx.ALIGN_CENTER)
        self.labelSizer.Add(self.maxLabel, 1, wx.EXPAND | wx.ALIGN_RIGHT)
        
        self.selectionSizer = wx.GridSizer(cols = 1)
        
        self.selectionLabel = wx.StaticText(self, -1, label = "", style = wx.ALIGN_CENTER)
        self.updateSelectionLabel()
        
        self.selectionSizer.Add(self.selectionLabel, 1, wx.EXPAND | wx.ALIGN_CENTER)
        
        self.sizer.Add(self.labelSizer, 1, wx.EXPAND)
        self.sizer.Add(self.slider, 1, wx.EXPAND)
        self.sizer.Add(self.selectionSizer, 1, wx.EXPAND)
        
        self.SetSizer(self.sizer)
        
        print "Min: %e, Max: %e" % (self.slider.GetMin(), self.slider.GetMax())
        
    def formatLabel(self, value):
        return (self.__labelFormat % (value))

    def updateValueLabel(self):
        self.valueLabel.SetLabel(self.formatLabel(self.getFloatValue()))
        self.valueLabel.SetWindowStyle(wx.ALIGN_CENTER)
        
        self.labelSizer.Layout()
        
    def updateSelectionLabel(self):
        try:
            floatStart = self.getFloatSelStart()
            floatEnd = self.getFloatSelEnd()
            format = "Selection: %s - %s" % (self.formatLabel(floatStart), self.formatLabel(floatEnd))
            
            self.selectionLabel.SetLabel(format)
            self.selectionLabel.SetWindowStyle(wx.ALIGN_CENTER)
            
            self.selectionSizer.Layout()
        except:
            pass
        
    def OnSliderChange(self, event):
        self.updateValueLabel()
        self.updateSelectionLabel()
        
        event.Skip()
        
    def getFloatValueBySliderValue(self, value):
        # Avoid rounding errors on the edge
        if value == self.slider.GetMin():
            return self.floatMin
        elif value == self.slider.GetMax():
            return self.floatMax
        
        if self.__useLogs:
            return math.exp(self.logMin + ((float(value) / self.slider.GetMax()) * (self.logMax - self.logMin)))
        else:
            return (self.floatMin + ((float(value) / self.slider.GetMax()) * (self.floatMax - self.floatMin)))
        
    def getSliderValueByFloatValue(self, value):
        return self.slider.GetMin() + (float(value) * self.slider.GetMax())
        
    def getFloatValue(self):
        return self.getFloatValueBySliderValue(self.slider.GetValue())
        
    def getFloatSelStart(self):
        return self.getFloatValueBySliderValue(self.slider.GetSelStart())
    
    def getFloatSelEnd(self):
        return self.getFloatValueBySliderValue(self.slider.GetSelEnd())    
        
    def OnLeftDown(self, event):
        self.previousStart = self.slider.GetValue()
        self.selecting = False
            
        event.Skip()
        
    def OnMotion(self, event):
        if event.Dragging():
            self.updateValueLabel()
            
            if event.ShiftDown():
                print "Motion with shift down!"
                self.selecting = True
                
            if self.selecting:
                self.setSelectionWithCurrentValue()
        
        event.Skip()
    
    def setSelectionWithCurrentValue(self):
        curVal = self.slider.GetValue()
        self.slider.SetSelection(min(self.previousStart, curVal), max(curVal, self.previousStart))
    
    def OnLeftUp(self, event):
        if self.selecting or event.ShiftDown():
            selStart = self.slider.GetSelStart()
            selEnd = self.slider.GetSelEnd()
            curVal = self.slider.GetValue()

            print "Updating selection!"
            self.setSelectionWithCurrentValue()
            
        self.selecting = False
        
        event.Skip()

class Filterable():
    def __init__(self):
        self.__propertyRanges = {}
        
    def setFilterRange(self, property, min, max):
        self.__propertyRanges[property] = (min, max)
        
    def getFilterRange(self, property):
        if property in self.__propertyRanges:
            return self.__propertyRanges[property]
        else:
            return None
        
    def setFilterRangeFromSlider(self, property, slider):
        return self.setFilterRange(property, slider.GetSelStart(), slider.GetSelEnd())
    
    def setFilterRangeFromLogSlider(self, property, logSlider):
        return self.setFilterRange(property, logSlider.getFloatSelStart(), logSlider.getFloatSelEnd())
    
    def inFilterRange(self, property, value):
        range = self.getFilterRange(property)
        if range is None:
            return False
        
        return (value >= range[0] and value <= range[1])