import math
import wx

class Color:
    RGB = 1
    RGB255 = 2
    HSV = 3
    
    # Type
    __type = None
    
    # Tokens (rgb or hsv)
    __t1 = None
    __t2 = None
    __t3 = None
    
    def __init__(self, type, t1 = None, t2 = None, t3 = None):
        self.__type = type
        self.__t1 = float(t1)
        self.__t2 = float(t2)
        self.__t3 = float(t3)
        
        if self.__type == Color.RGB255:
            divisor = float(255)
            self.__t1 /= divisor
            self.__t2 /= divisor
            self.__t3 /= divisor
            self.__type = Color.RGB
        
    def getT1(self):
        return self.__t1
    
    def getT2(self):
        return self.__t2
    
    def getT3(self):
        return self.__t3
    
    def getRGB(self):
        return self.getTuple()
    
    def getHSV(self):
        return self.getTuple()
    
    def getTuple(self):
        return (self.getR(), self.getG(), self.getB())
    
    def toRGB(self):
        if self.getType() == Color.RGB:
            return self
        
        # 0 <= hue < 360
        hue = self.getH()
        
        # 0 <= sat <= 1
        # 0 <= val <= 1
        sat = self.getS()
        val = self.getV()
        
        # Reference: http://en.wikipedia.org/wiki/HSV_color_space
        Hi = math.floor(hue / 60.0) % 6
        f = (hue / 60.0) - Hi
        p = val * (1.0 - sat)
        q = val * (1.0 - f * sat)
        t = val * (1 - (1 - f) * sat)
        
        if (Hi == 0):
            reTuple = (val, t, p)
        elif (Hi == 1):
            reTuple = (q, val, p)
        elif (Hi == 2):
            reTuple = (p, val, t)
        elif (Hi == 3):
            reTuple = (p, q, val)
        elif (Hi == 4):
            reTuple = (t, p, val)
        elif (Hi == 5):
            reTuple = (val, p, q)
        
        re = Color(Color.RGB, reTuple[0], reTuple[1], reTuple[2])
        return re
    
    def toRGB255(self):
        use = self
        if use.getType() == Color.HSV:
            use = self.toRGB()
            
        if use.getType() == Color.RGB:
            return Color(Color.RGB255, self.getR() * 255, self.getG() * 255, self.getB() * 255)
        else:
            return use
    
    def toHSV(self):
        if self.getType() == Color.HSV:
            return self
        
        red = self.getR()
        blue = self.getB()
        green = self.getG()
        
        hue = None
        sat = None
        val = None
        
        minRGB = min(red, green, blue)
        maxRGB = max(red, green, blue)
        
        val = float(maxRGB)
        delta = float(maxRGB - minRGB)
        
        if maxRGB != 0:
            sat = delta / maxRGB
        else:
            # Can't translate!
            sat = 0.0
            hue = 0.0
        
        if delta != 0:
            if red == maxRGB:
                hue = (green - blue) / delta
            elif green == maxRGB:
                hue = 2 + (blue - red) / delta
            else:
                hue = 4 + (red - green) / delta
        else:
            hue = 0.0
            
        hue *= 60
        if hue < 0:
            hue += 360
           
        return Color(Color.HSV, hue, sat, val)
    
    def getR(self):
        return self.getT1()
    
    def getH(self):
        return self.getR()
    
    def getG(self):
        return self.getT2()
    
    def getS(self):
        return self.getG()
    
    def getB(self):
        return self.getT3()
    
    def getV(self):
        return self.getB()
    
    def setR(self, t):
        self.setT1(t)
        
    def setH(self, t):
        self.setT1(t)
        
    def setG(self, t):
        self.setT2(t)
        
    def setS(self, t):
        self.setT2(t)
        
    def setB(self, t):
        self.setT3(t)
        
    def setV(self, t):
        self.setT3(t)
        
    def setT1(self, t):
        self.__t1 = t
        
    def setT2(self, t):
        self.__t2 = t
        
    def setT3(self, t):
        self.__t3 = t
    
    def getType(self):
        return self.__type
    
    def provideToFunction(self, func, toRGB = True):
        provide = self
        if toRGB and self.getType() != Color.RGB:
            provide = self.toRGB()
        
        return func(provide.getT1(), provide.getT2(), provide.getT3())
    
    def getWxColour(self):
        use = self
        if self.getType() != Color.RGB:
            use = self.toRGB()
        
        return wx.Colour(use.getR() * 255, use.getG() * 255, use.getB() * 255)
    
    def provideToFunctionAsWxColour(self, func):           
        clr = self.getWxColour()
        return func(clr)
        
    def getUBString(self):
        use = self
        if self.getType() != Color.RGB:
            use = self.toRGB()
            
        imgStr = []
        for item in use.getTuple():
            imgStr.append(chr(item * 255)) 
     
        return ''.join(imgStr)
        
    def __str__(self):
        return "Color (" + str(self.getT1()) + ", " + str(self.getT2()) + ", " + str(self.getT3()) + ")"