import wx

from util.alignmentmanager import *

from display.wxhelper import *
from display.colorkey import *
from display.applicationhelper import *

class ColorKeyView(wx.Panel, SequenceSwapable):
    def __init__(self, parent, id, pos = wx.DefaultPosition, size = wx.DefaultSize, orientation = wx.HORIZONTAL, seqOrder = None, aln = None):
        self.__init = False
        wx.Panel.__init__(self, parent, id, pos, size)
        SequenceSwapable.__init__(self, seqOrder)
        
        self.__aln = aln
        self.__colorKey = getColorKey(self.__aln)
        
        self.__numColors = self.__colorKey.getNumColors()
        
        self.__keySizer = wx.GridSizer(cols = self.__numColors, vgap = 0, hgap = 2)
        
        for j in range(int(len(seqOrder) / self.__numColors) + 1): # for each row
            iStart = j * self.__numColors
            iEnd = iStart + self.__numColors
            if iEnd > len(seqOrder):
                iEnd = len(seqOrder)
            
            for i in range(iStart, iStart + self.__numColors):
                iOrder = None
                try:
                    iOrder = seqOrder[i]
                except:
                    self.__keySizer.Add((0, 0), 0) # add spacer if nothing here
                    continue
                
                clr = self.__colorKey.get(iOrder)
                
                keyUnit = ColorKeyViewColorKey(self, -1, color = clr[int(len(clr) / 2)])
                keyUnit.Bind(wx.EVT_LEFT_DOWN, self.OnLeftDown)
                keyUnit.Bind(wx.EVT_LEFT_UP, self.OnLeftUp)
                
                self.__keySizer.Add(keyUnit, 1, wx.EXPAND)
    
            for i in range(iStart, iEnd):
                iOrder = seqOrder[i]
                
                label = aln.getSeqDescriptionByIndex(iOrder)
                
                labelCtrl = wx.StaticText(self, -1, label)
                labelCtrl.Bind(wx.EVT_LEFT_DOWN, self.OnLeftDown)
                labelCtrl.Bind(wx.EVT_LEFT_UP, self.OnLeftUp)
                
                self.__keySizer.Add(labelCtrl, 0, wx.ALIGN_CENTER)
        
        # Setup main sizer
        self.__mainSizer = wx.FlexGridSizer(rows = 1, hgap = 4)

        # Setup key label sizer
        self.__keyLabelSizer = wx.BoxSizer(wx.VERTICAL)
        
        wxHelper = getWxHelper()
        keyLabel = wxHelper.makeBoldStaticText(self, -1, "Sequence Color Key:")
        self.__keyLabelSizer.Add(keyLabel, 1, wx.EXPAND)
        
        if self.__aln is None:
            self.__keyLabelSizer.Add((0, 0), 0)
        else:
            dragHelpLabel = wx.StaticText(self, -1, "Drag color boxes to change order")
            self.__keyLabelSizer.Add(dragHelpLabel, 1, wx.EXPAND)    
        
        self.__mainSizer.Add(self.__keyLabelSizer, 0)
        
        self.__mainSizer.Add(self.__keySizer, 0)
        
        # Take up extra space on the right
        self.__mainSizer.Add(wx.Window(self, -1), 1, wx.EXPAND)
        self.__mainSizer.AddGrowableCol(2)
        
        self.__dragColumn = None
        
        self.SetSizer(self.__mainSizer)
        
        self.__init = True
        
    def getColumnByObject(self, obj):
        i = 0
        index = None
        for objSeek in self.__keySizer.GetChildren():
            window = objSeek.GetWindow()
            if window is not None and window == obj:
                index = i
                break
                
            i += 1
        
        index = self.getColumnFromPosition(index)
        
        return self.seqOrder[index]
    
    def getColumnFromPosition(self, position):
        if (int(position / self.__numColors)) % 2 == 1: # if we're in a label row
            position -= self.__numColors
        
        position = (((int(position / self.__numColors) / 2) * self.__numColors) + (position % self.__numColors))
        
        return position    
    
    def getPositionFromColumn(self, col):
        return (int(col / self.__numColors) * 2 * self.__numColors) + (col % self.__numColors)
    
    def OnLeftDown(self, event):
        self.__dragColumn = self.getColumnByObject(event.GetEventObject())
    
        frame = getApplicationFrame()
        frame.setNotifySequence(self.__aln, self.seqOrder[self.__dragColumn])
    
    def OnLeftUp(self, event):
        if self.__dragColumn is None:
            return
        
        dropColumn = self.getColumnByObject(event.GetEventObject())
        if dropColumn is None:
            return
        
        if self.__dragColumn != dropColumn:
            self.swapSequenceOrder(self.__dragColumn, dropColumn)
            
        self.__dragColumn = None
            
    def swapSequenceOrder(self, one, two, fromApplication = False):
        if not self.__init or self.__aln is None:
            return
        
        if not SequenceSwapable.orderChanged(self, one, two):
            return
        
        if fromApplication:
            # Only change things if it comes from the application
            SequenceSwapable.swapSequenceOrder(self, one, two, fromApplication)
            self.swapRowOrder(one, two)
        else:
            # If something screws up, i.e., we're moving a feature track, ignore
            # the error and just switch them
            #try:
            if True:
                frame = getApplicationFrame()
                frame.swapSequenceOrderForAlignment(self.__aln, one, two)
            #except:
            #    self.swapRowOrder(one, two)
    
    def swapRowOrder(self, one, two):
        if not self.__init:
            return
        
        alnNumSeqs = self.__aln.getNumSequences()
        
        oneAdj = self.getPositionFromColumn(min(one, two))
        twoAdj = self.getPositionFromColumn(max(two, one))
        
        oneDisplay = self.__keySizer.GetItem(oneAdj).GetWindow()
        oneLabel = self.__keySizer.GetItem(oneAdj + self.__numColors).GetWindow()
        
        twoDisplay = self.__keySizer.GetItem(twoAdj).GetWindow()
        twoLabel = self.__keySizer.GetItem(twoAdj + self.__numColors).GetWindow()

        # Detach everything: order not important here
        self.__keySizer.Detach(oneDisplay)
        self.__keySizer.Detach(oneLabel)
        self.__keySizer.Detach(twoDisplay)
        self.__keySizer.Detach(twoLabel)

        # Reattach everything
        # The order is important here (smallest to largest)
        winsToAttach = {
                        oneAdj: twoDisplay,
                        twoAdj: oneDisplay,
                        (oneAdj + self.__numColors): twoLabel,
                        (twoAdj + self.__numColors): oneLabel
                    }
        
        order = winsToAttach.keys()
        order.sort()
        for i in order:
            win = winsToAttach[i]
            
            prop = 0
            flags = wx.ALIGN_CENTER
            
            if isinstance(win, ColorKeyViewColorKey):
                prop = 1
                flags |= wx.EXPAND
                
            self.__keySizer.Insert(i, win, prop, flags)
        
        self.__keySizer.Layout()
        
        oneLabel.Refresh()
        twoLabel.Refresh()
        oneDisplay.Refresh()
        twoDisplay.Refresh()
        
class ColorKeyViewColorKey(wx.Panel):
    def __init__(self, parent, id, pos = wx.DefaultPosition, size = wx.DefaultSize, color = None):
        wx.Panel.__init__(self, parent, id, pos, size)
        
        self.__color = color
        
        self.Bind(wx.EVT_PAINT, self.OnPaint)
        self.Bind(wx.EVT_SIZE, self.OnSize)
        
    def OnPaint(self, event):
        dc = wx.PaintDC(self)
        
        size = self.GetClientSize()

        rect = wx.Rect(0, 0, size.width, size.height)
        
        helper = getWxHelper()        
        helper.drawFilledRectangle(dc, rect, self.__color)
        
    def OnSize(self, event):
        self.Refresh()
