import wx

from display.applicationhelper import *
from display.wxhelper import *
from display.openglhelper import *

from util.preferences import * 

from OpenGL.GL import *
from OpenGL.GLU import *
from OpenGL.GLUT import *

from struct import pack, unpack
    
class RectangleDrawer(Zoomable, wx.Timer, SequenceNotifiable):
    HORIZONTAL = 1
    VERTICAL = 2
    
    def __init__(self, orientation = None, zoom = None, order = None, drawBorder = False):
        self.__init = False
        Zoomable.__init__(self, zoom)
        SequenceNotifiable.__init__(self)
        wx.Timer.__init__(self)
        
        if orientation is None:
            orientation = RectangleDrawer.HORIZONTAL
        self.__orientation = orientation
    
        self.__rects = None
        
        self.__drawBorder = drawBorder
        
        self.__colors = None
        self.__altColors = None
        
        self.__order = None
        
        self.__boxes = {}
        
        self.__positionValues = None
        
        self.__rectWidth = 1
        self.__rectHeight = 1
        
        self.__x = 0
        self.__y = 0
        
        self.__timerTimes = None
        self.__flashing = False
        self.__flashingIndex = None
        
        self.__refreshCallback = None
        
        self.__seqToList = {}
        
        self.setDirty()
        
        self.__init = True
        
    def getListForSeq(self, seq):
        if seq not in self.__seqToList:
            self.__seqToList[seq] = glGenLists(1)
        
        return self.__seqToList[seq]
    
    def swapRowOrder(self, one, two):
        valueOne = self.__order[one]
        valueTwo = self.__order[two]
        
        self.__order[one] = valueTwo
        self.__order[two] = valueOne
        
    def setupVariables(self):
        pass
    
    def setDirty(self, dirty = True):
        self.__dirty = dirty
    
    def setZoom(self, start, end, fromApplication = False):
        Zoomable.setZoom(self, start, end, fromApplication)
        
        # We don't need to set dirty anymore
        #if self.__init:
        #    self.setDirty()
        
    def setColors(self, colors):
        self.__colors = colors
        self.setDirty()
        
    def setAltColors(self, colors):
        self.__altColors = colors
        self.setDirty()

    def setPositionValues(self, positionValues):
        self.__positionValues = positionValues
        
        # Assume things were put in order in the first place
        if self.__order is None:
            self.__order = range(len(self.__positionValues))
        
        self.setDirty()
        
    def setPosition(self, x, y):
        self.__x = x
        self.__y = y
    
    def getOrientation(self):
        return self.__orientation
        
    def drawRectangles(self, *args):            
        if self.start is None or self.end is None:
            self.setZoom(0, len(self.__positionValues))
       
        self.drawRectanglesGL(*args)
        
    def drawRectanglesGL(self):
        eachHeight = float(self.__rectHeight) / len(self.__order)
        eachWidth = float(self.__rectWidth) / len(self.__order)
        
        for i in self.__order:
            if self.__dirty:
                self.__boxes[i] = []
                firstCoordinate = None                
                
                listID = self.getListForSeq(i)
                
                glNewList(listID, GL_COMPILE)
                glShadeModel(GL_FLAT)
                
                glBegin(GL_QUAD_STRIP)
                
                if self.getOrientation() == RectangleDrawer.HORIZONTAL:
                    glVertex2f(self.__x, self.__y)
                    glVertex2f(self.__x, self.__y + eachHeight)
                else:
                    glVertex2f(self.__x, self.__y)
                    glVertex2f(self.__x + eachWidth, self.__y)
                
                for j in range(len(self.__positionValues[i])):
                    posValue = self.__positionValues[i][j]
                    
                    if posValue is None:
                        continue
    
                    color = None
                    if posValue < 0:
                        if firstCoordinate is not None:
                            self.__boxes[i].append((firstCoordinate, j))
                            firstCoordinate = None
                        
                        rounded = posValue * len(self.__altColors)
                        rounded = int(rounded - 0.5)
                        
                        color = self.__altColors[min(-rounded, len(self.__altColors) - 1)]
                    else:
                        if firstCoordinate is None:
                            firstCoordinate = j
                             
                        rounded = posValue * len(self.__colors[i])
                        rounded = int(rounded + 0.5)
                        
                        color = self.__colors[i][min(rounded, len(self.__colors[i]) - 1)]
                                  
                    color.provideToFunction(glColor3f)
                    if self.getOrientation() == RectangleDrawer.HORIZONTAL:
                        glVertex2f((j + 1) * self.__rectWidth + self.__x, self.__y)
                        glVertex2f((j + 1) * self.__rectWidth + self.__x, self.__y + eachHeight)                    
                    else:
                        glVertex2f(self.__x, self.__y + (j + 1) * self.__rectHeight)
                        glVertex2f(self.__x + eachWidth, self.__y + (j + 1) * self.__rectHeight)
                
                glEnd()
                
                if firstCoordinate is not None:
                    self.__boxes[i].append((firstCoordinate, j))
                
                if self.__drawBorder:
                    glDisable(GL_BLEND)
                    glDisable(GL_LINE_SMOOTH)
                    glLineWidth(1.0)
                    
                    grayIntensity = 93
                    glColor3f(grayIntensity / 256.0, grayIntensity / 256.0, grayIntensity / 256.0)
                    
                    glBegin(GL_LINES)
                    if self.getOrientation() == RectangleDrawer.HORIZONTAL:
                        glVertex2f(self.__x, self.__y + eachHeight)
                        glVertex2f(self.__x + self.__rectWidth * len(self.__positionValues[i]), self.__y + eachHeight)       
                    else:
                        glVertex2f(self.__x + eachWidth, self.__y)
                        glVertex2f(self.__x + eachWidth, self.__y + self.__rectHeight * len(self.__positionValues[i]))
                    glEnd()
                
                glEndList()
            
        glMatrixMode(GL_MODELVIEW)
        glPushMatrix()
        
        newOrder = list(self.__order)
        newOrder.reverse() # draw in reverse order
        
        if self.getOrientation() == RectangleDrawer.HORIZONTAL:
            glTranslatef(-(self.start * self.__rectWidth), eachHeight * (len(newOrder) - 1), 0)
        else:
            glTranslatef(eachWidth * (len(newOrder) - 1), -(self.start * self.__rectHeight), 0)
        
        for i in newOrder:
            listID = self.getListForSeq(i)
            
            glCallList(listID)
            if self.__flashing and self.__flashingIndex == i:
                self.drawFlashBoxes(self.__boxes[i])
                print "Drawing flash box for %d" % (i)
                print "Flashing boxes: %s" % (str(self.__boxes[i]))
            
            if self.getOrientation() == RectangleDrawer.HORIZONTAL:
                glTranslatef(0, -eachHeight, 0)
            else:
                glTranslatef(-eachWidth, 0, 0)
                
        glMatrixMode(GL_MODELVIEW)
        glPopMatrix()
        
        self.setDirty(False)
        
    def drawFlashBoxes(self, boxes):
        # Draw black box around positive position values for index
        eachHeight = float(self.__rectHeight) / len(self.__order)
        eachWidth = float(self.__rectWidth) / len(self.__order)        

        black = Color(Color.RGB, 0, 0, 0)
        black.provideToFunction(glColor3f)
        
        for box in boxes:
            if self.getOrientation() == RectangleDrawer.HORIZONTAL:
                print "Rect: %s" % (str((self.__x + (box[0] * eachWidth), self.__y, self.__x + (box[1] * eachWidth), self.__y + eachHeight)))
                glRectf(self.__x + (box[0] * self.__rectWidth), self.__y, self.__x + (box[1] * self.__rectWidth), self.__y + eachHeight)
            else:
                glRectf(self.__x, self.__y + (box[0] * self.__rectHeight), self.__x + eachWidth, self.__y + (box[1] * self.__rectHeight))
                
    def setNotifySequence(self, index, fromApplication = False):
        self.__flashingIndex = index
        
        preferences = getPreferences()
        self.__timerTimes = preferences.getNumberOfNotifyFlashes()
        
        print "Trying to flash %d" % (index)
        self.Start(preferences.getNotifyTimerFlashDelay(), False)
    
    def Notify(self):
        print "Notify call!"
        self.__flashing = not self.__flashing
        
        if self.__refreshCallback is not None:
            (self.__refreshCallback)()
        
        self.__timerTimes -= 1
        if self.__timerTimes <= 0:
            self.Stop()
            self.__flashing = False
            
    def setRefreshCallback(self, callback):
        self.__refreshCallback = callback