#from Bio import Clustalw
from Bio.SubsMat import MatrixInfo
from Bio import SeqIO

from util.preferences import  *

from display.wxhelper import *
from display.applicationhelper import *

class AlignmentManager:
    INVALID = 0
    CLUSTALW = 1
    PHYLIP = 2
    
    def __init__(self):
        self.__alignment = None
        self.__numSequences = 0
        self.__alnLength = None
        
        self.__alnScores = {}
        
        self.__uniqueFeatureObjects = []
        
        self.__title = ""
        
        self.__matrix = MatrixInfo.blosum62
    
    def loadAlignmentFromFile(self, type, file):
        # Reference: http://biopython.org/wiki/SeqIO
        handle = open(file, "rU")
        textType = None
        
        if type == AlignmentManager.CLUSTALW:
            textType = "clustal"
        elif type == AlignmentManager.PHYLIP:
            textType = "phylip"
        else:
            raise NotImplementedError("Invalid type")
        
        self.__alignment = SeqIO.to_alignment(SeqIO.parse(handle, textType))
        
        self.__fetcherQueriedStates = {}
        
        self.__numSequences = len(self.__alignment.get_all_seqs())
        self.__alnLength = None
        
        self.__title = file
        
        self.__cleanupSequences()        
    
    def getTitle(self):
        return self.__title
    
    def __cleanupSequences(self):
        # Remove silly stuff that Jalview adds
        for seqRecord in self.getAllSeqRecords():
            parts = seqRecord.name.partition('/')
            seqRecord.name = parts[0]
            
            parts = seqRecord.id.partition('/')
            seqRecord.id = parts[0]
    
    def scoreByMatrix(self, t1, t2):
        if t1 == '-' or t2 == '-':
            return 0
        
        tuple = (t1, t2)
        if tuple in self.__matrix:
            return self.__matrix[tuple]
        
        tuple = (t2, t1)
        if tuple in self.__matrix:
            return self.__matrix[tuple]
    
    def scoreByIdentity(self, t1, t2):
        # Match only on things that are identical and not gaps
        if t1 == t2 and t1 != '-' and t2 != '-':
            return 1
        else:
            return 0
    
    def scoreAlignment(self):
        preferences = getPreferences()
        prefScoreMethod = preferences.getScoreMethod()
        
        # Beyond reasonable limits
        minScore = 100000
        maxScore = -100000
        
        for i in range(self.getLength()):
            self.__alnScores[i] = 0
            
            column = self.getColumn(i)
            
            for j in range(len(column)):
                for k in range(len(column)):
                    if j == k:
                        continue
                    
                    if prefScoreMethod == Preferences.SCORE_METHOD_BLOSUM62:
                        self.__alnScores[i] += self.scoreByMatrix(column[j], column[k])
                    elif prefScoreMethod == Preferences.SCORE_METHOD_IDENTITY:
                        self.__alnScores[i] += self.scoreByIdentity(column[j], column[k])
            
            if self.__alnScores[i] > maxScore:
                maxScore = self.__alnScores[i]
            if self.__alnScores[i] < minScore:
                minScore = self.__alnScores[i]
                
        # Scale everything nicely
        if minScore < 0:
            maxScore += -minScore
            
        if maxScore == 0:
            # Bit of a hack, but we don't want divide by zeros
            maxScore = 0.0001
            
        for i in range(self.getLength()):
            # [0, inf]
            if minScore < 0:
                self.__alnScores[i] += -minScore
            
            # [0, 1]
            self.__alnScores[i] /= float(maxScore)
                
    def clearColumnScores(self):
        self.__alnScores = {}
                
    def getColumnScore(self, col):
        if len(self.__alnScores) == 0:
            self.scoreAlignment()
            
        if col in self.__alnScores:
            return self.__alnScores[col]
        else:
            return None
    
    def getAllSeqRecords(self):
        return self.__alignment.get_all_seqs()
    
    def getSeqByIndex(self, index):
        list = self.getAllSeqs()
        return list[index]
    
    def getSeqRecordByIndex(self, index):
        list = self.getAllSeqRecords()
        return list[index]
    
    def getAllSeqs(self):
        re = []
        for seqRecord in self.__alignment.get_all_seqs():
            re.append(seqRecord.seq)
        
        return re
    
    def getColumn(self, col):
        return list(self.__alignment.get_column(col))
    
    def getLength(self):
        # Cache this, as it could be CPU intensive to do it the
        # number of times we need to
        if self.__alnLength is None:
            self.__alnLength = self.__alignment.get_alignment_length()
        
        return self.__alnLength
    
    def getAlignment(self):
        return self.__alignment
    
    def getSeqStringByIndex(self, index, gaps = True):
        seq = self.getSeqByIndex(index)
        str = seq.tostring()
        
        if gaps == False:
            str = str.replace("-", "")
            
        return str
    
    def getNumSequences(self):
        return self.__numSequences
    
    def setFetcherQueriedState(self, seqRecord, fetcher, state):
        if seqRecord not in self.__fetcherQueriedStates:
            self.__fetcherQueriedStates[seqRecord] = {}
            
        self.__fetcherQueriedStates[seqRecord][fetcher] = state
        
    def getFetcherQueriedState(self, seqRecord, fetcher):
        if seqRecord not in self.__fetcherQueriedStates:
            return False
        
        if fetcher not in self.__fetcherQueriedStates[seqRecord]:
            return False
        
        return self.__fetcherQueriedStates[seqRecord][fetcher]
    
    def fetchFeaturesForSeqRecord(self, featFetchList, seqRecord, onlyNew = True):
        helper = getWxHelper()
        frame = getApplicationFrame()
        preferences = getPreferences()
        
        for fetcher in featFetchList:
            if onlyNew and self.getFetcherQueriedState(seqRecord, fetcher):
                continue
            
            tries = preferences.getFetcherRetries()
            while tries > 0:
                features = []
                try:
                    features = fetcher.fetch(seqRecord.seq.tostring())
                except:
                    # Display a message and go on
                    tries -= 1
                    if tries == 0:
                        helper.displayErrorMessage(frame, "Unable to fetch features from '%s' for sequence '%s'\n\nInformation will not be complete.  Please try again later." % (fetcher.getName(), self.getSeqDescriptionBySeqRecord(seqRecord)), "Fetcher error")
                        
                    continue
                
                for feat in features:
                    seqRecord.features.append(feat)
                
                # Sort by e-value (most likely)
                seqRecord.features.sort()
                
                self.setFetcherQueriedState(seqRecord, fetcher, True)
                break
                
    def fetchFeatures(self, featFetchList, onlyNew = True):
        frame = getApplicationFrame()
        
        for i in range(self.getNumSequences()):
            seqRecord = self.getSeqRecordByIndex(i)
            self.fetchFeaturesForSeqRecord(featFetchList, seqRecord, onlyNew)
            
            frame.setFetcherProgress(i)
    
    def getSeqDescriptionByIndex(self, index): 
        seqRecord = self.getSeqRecordByIndex(index)
        return self.getSeqDescriptionBySeqRecord(seqRecord)
        
    def getSeqDescriptionBySeqRecord(self, seqRecord):
        if seqRecord.name is not None and seqRecord.name != '' and seqRecord.name != '<unknown name>':
            return seqRecord.name
        if seqRecord.id is not None and seqRecord.id != '' and seqRecord.id != '<unknown id>':
            return seqRecord.id
        else:
            return str(index)
    
    def getUniqueFeatureData(self):
        evalues = {}
        numbereValueFeats = {}
        numberPresent = {}
        
        uniqueFeatures = {}
        
        if len(self.__uniqueFeatureObjects) == 0:
            for seqRecord in self.getAllSeqRecords():
                for feature in seqRecord.features:
                    if not feature.isTrackFeature():
                        continue
                    
                    id = feature.id
                    
                    if 'e-value' in feature.qualifiers:                        
                        if id in evalues:
                            evalues[id] += float(feature.qualifiers['e-value'])
                        else:
                            evalues[id] = float(feature.qualifiers['e-value'])
                    
                        if id in numbereValueFeats:
                            numbereValueFeats[id] += 1
                        else:
                            numbereValueFeats[id] = 1
                    
                    # Count how many sequences have this feature
                    if id in numberPresent:
                        numberPresent[id] += 1
                    else:
                        numberPresent[id] = 1
                    
                    if id in uniqueFeatures:
                        # No need to dup
                        continue
                    
                    name = None
                    if 'name' in feature.qualifiers:
                        name = feature.qualifiers['name']
                    else:
                        name = str(id)
                        
                    desc = None
                    if 'description' in feature.qualifiers:
                        desc = feature.qualifiers['description']
                    else:
                        desc = ''
                    
                    dbID = None
                    if 'dbID' in feature.qualifiers:
                        dbID = feature.qualifiers['dbID']
                    else:
                        dbID = ''
                        
                    url = None
                    if 'url' in feature.qualifiers:
                        url = feature.qualifiers['url']
                    else:
                        url = ''
                        
                    # Tuple
                    uniqueFeatures[id] = (name, desc, dbID, url)
            
            for featID in uniqueFeatures:
                evalue = -1
                if featID in evalues:
                    evalue = evalues[featID] / numbereValueFeats[featID]
                
                tuple = uniqueFeatures[featID]
                obj = UniqueFeatureData(featID, tuple[0], tuple[1], tuple[2], tuple[3], evalue, numberPresent[featID])
                
                self.__uniqueFeatureObjects.append(obj)
            
            self.__uniqueFeatureObjects.sort()
            
        return self.__uniqueFeatureObjects
    
    def getNonTrackFeaturesByIndex(self, index, type = None):
        re = []
        
        seqRecord = self.getSeqRecordByIndex(index)
        for feat in seqRecord.features:
            if feat.isTrackFeature():
                continue
            
            if type is None or feat.type == type:
                re.append(feat)
            
        return re
    
    def getFeatureFromIndexAndID(self, seqIndex, featureID):
        seqRecord = self.getSeqRecordByIndex(seqIndex)
        featureID = featureID
        
        if seqRecord is not None:
            for feature in seqRecord.features:
                if feature.id == featureID:
                    return feature
                
        return None
    
class UniqueFeatureData():
    def __init__(self, id, name, description, dbID, url, evalue, numberPresent):
        self.id = id
        self.name = name
        self.description = description
        self.dbID = dbID
        self.url = url
        self.evalue = evalue
        self.numberPresent = numberPresent
        
    def __cmp__(self, other):
        if self.evalue >= 0 and other.evalue >= 0:
            return cmp(self.evalue, other.evalue)
        else:
            return cmp(self.name, other.name)
        
    def __str__(self):
        return "UniqueFeatureData for %s (%s [%s]) with evalue %.2e" % (self.id, self.name, self.description, self.evalue)
    
aln = None
def setAlignment(alignment):        
    global aln
    aln = alignment
    
def getAlignment():
    global aln
    return aln            